/*
 * Copyright 1996-2002 netFluid Technology Ltd
 * http://www.nfluid.co.uk
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * $Revision: 1.1.1.1 $  $Date: 2003/01/10 23:16:04 $
 *
 * Author: David Zambonini
 *
 * ---------------------------------------------------------------------------
 */

#include <phlib/memdbinc.h>

/* ------------------------------------------------------------------------- */
/* Debugging version of realloc 
 */
void *
realloc_debug(const char *filename,unsigned line,void *block,size_t size)
{
  /* Perform a free if it's a deallocation request */
  if (size==0) {
    free_debug(filename,line,block);
    return NULL;
  }

  /* Perform a malloc if there's no original block */
  if (block==NULL) {
    return malloc_debug(filename,line,size);
  }

  /* We have used a debugging command */
  set_mem_db_used();

  if (size>max_alloc) {
    fprintf(stderr,"realloc size overrun in %s, line %i\n",filename,line);
  } else {
    register char *ptr;

      /* Dereference pointer */
    block=(void*)(((char*)block)-sizeof(mem_db_info));

    /* Must check tag ID here */
    check_mem_db_info((mem_db_info*)block,filename,line);

    /* Do realloc here */
    ptr=(char*)realloc(block,size+sizeof(mem_db_info));

    if (ptr!=NULL) {
      move_mem_db_info((mem_db_info*)block,(mem_db_info*)ptr,"realloc",
		       size,filename,line);
      return (void*)(((char*)ptr)+sizeof(mem_db_info));
    }
  }

  /* Failure to allocate */
  set_mem_db_failure();
  return NULL;
}

/* ------------------------------------------------------------------------- */
/* end */
